
#include "intern.h"

struct {
   GPDRAWSURFACE *draw;
   const unsigned char *charset;
   unsigned char fc;
   unsigned char bc;
   unsigned char x;
   unsigned char y;
} SOMFont_console;


void SOMFontConsoleCursor( short x, short y )
{
   if(x>=0) SOMFont_console.x = x;
   if(y>=0) SOMFont_console.y = y;
}


static inline void SOMFontConsoleScroll()
{
   unsigned char *screen = SOMFont_console.draw->o_buffer;
   
   int x,y;
   unsigned long *s, *d;
   
   while(SOMFont_console.y >= 30)
   {
      for (x = 0; x < 320; x++)
      {
         s = (unsigned long *) &screen[(x*240) + 228];
         d = (unsigned long *) &screen[(x*240) + 236];
         
         for(y = 0; y < 232; y+=sizeof(unsigned long))
         {
            *d-- = *s--;
         }
      }
      
      GpRectFill( NULL, SOMFont_console.draw, 0, SOMFont_console.draw->buf_h - 8,
                  SOMFont_console.draw->buf_w, 8, SOMFont_console.bc );
      SOMFont_console.y--;
   }
}

void SOMFontConsoleInit( GPDRAWSURFACE *gpDraw, const unsigned char *charset,
                         unsigned char fc, unsigned char bc )
{
   if( gpDraw )
   {
      SOMFont_console.draw = gpDraw;
   }
   else
   {
      /* maybe init own screen here? but now just quit */
      return;
   }
   SOMFontConsoleCharset( charset );
   SOMFontConsoleColor( fc );
   SOMFontConsoleClear( bc );
}

void SOMFontConsoleCharset( const unsigned char *charset )
{
   if(charset)
   {
      SOMFont_console.charset = charset;
   }
   else
   {
      SOMFont_console.charset = SOMFont_default_charset;
   }
}

void SOMFontConsolePrint( const unsigned char *message )
{
   unsigned char transparentC;
   
   transparentC = SOMFontFindNonTransColor(SOMFont_console.fc,SOMFont_console.bc);
   
   while(*message)
   {
      switch(*message)
      {
         case '\n':
            SOMFont_console.y++;
            /*slip through*/
         case '\r':
            SOMFont_console.x = 0;
            break;
         default:
            SOMFontConsoleScroll();
            SOMFontDrawChar( SOMFont_console.draw,
                             SOMFont_console.x * 8,
                             SOMFont_console.y * 8,
                             SOMFont_console.charset,
                             *message,
                             SOMFont_console.fc,
                             SOMFont_console.bc,
                             transparentC );
            if(++SOMFont_console.x >= 40)
            {
               SOMFont_console.x = 0;
               SOMFont_console.y++;
            }
      }
      message++;
   }
}

void SOMFontConsoleClear( unsigned char bc )
{
   SOMFont_console.bc = bc;
   SOMFont_console.x = 0;
   SOMFont_console.y = 0;
   GpRectFill( NULL, SOMFont_console.draw, 0, 0,
               SOMFont_console.draw->buf_w, SOMFont_console.draw->buf_h, bc );
}

void SOMFontConsoleColor( unsigned char fc )
{
   SOMFont_console.fc = fc;
}

