
#include "intern.h"



unsigned char SOMFontFindNonTransColor( unsigned char fc, unsigned char bg )
{
   unsigned char tc;
   
   tc = 0xBD;
   if( (fc == tc) || (bg == fc) )
   {
      tc = 0xAD;
      if( (fc == tc) || (bg == fc) )
      {
         tc = 0x9D;
      }
   }
   
   return tc;
}



void SOMFontDrawChar(GPDRAWSURFACE *ds, int x, int y, 
   const unsigned char *cs, unsigned char ch,
   unsigned char fc, unsigned char bc, unsigned char tc)
{
   /*
    * draw character with single height (8x8) on 16bit framebuffer
    *
    * ds: pointer to GPDRAWSURFACE
    * x:  xpos in pixels
    * y:  ypos in pixels
    * cs: pointer to charset
    * ch: char to draw
    * fc: foreground color
    * bc: background color
    * tc: transparent color
    */
   int line,pixel;
   unsigned char bitmap[8];
   const unsigned char *src;

   if(ch < 0x20) ch=' ';
  
   src = cs;
   src += 8 * (ch-' ');
   for(line = 0; line < 8; line++)
   {
      bitmap[line] = (ch == ' ') ? 0 : *src;
      src++;
   }

   for(pixel = 0; pixel < 8; pixel++)
   {
      if((x+pixel >= 0) && (x+pixel < 320))
      {
         for(line = 7; line >= 0; line--)
         {
            if((y+line >= 0) && (y+line < 240))
            {
               if((1 << (7-pixel) & bitmap[line]))
               {
                  if(fc != tc) GpPointSet(ds, x+pixel, y+line, fc);
               }
               else
               {
                  if(bc != tc) GpPointSet(ds, x+pixel, y+line, bc);
               }
            }
         }
      }
   }
}



void SOMFontFastString(GPDRAWSURFACE *ds, int x, int y, 
   const unsigned char *cs, const unsigned char *string,
   unsigned char fc, unsigned char bc)
{
   /*
    * draw character with single height (8x8) on 16bit framebuffer
    *
    * ds: pointer to GPDRAWSURFACE
    * x:  xpos in pixels
    * y:  ypos in pixels (needs to be multiple of 4)
    * cs: pointer to charset
    * ch: chars to draw (terminated with ('\0'));
    * fc: foreground color
    * bc: background color
    */
   int line, pixel;
   register unsigned int cd1, cd2, vd1, vd2;
   register unsigned int *vram;
   const unsigned char *src;
   register unsigned char ch;

   while(*string)
   {
      ch = *(string++);
      
      if(*cs < 7)
      {
         if((ch < 0x20) || (ch > 0x7f)) ch=' ';
         if((*cs ==  2) && (ch > 0x5f)) ch-=0x20;
      }
      
      if(ch == ' ')
      {
         cd1 = 0;
         cd2 = 0;
      }
      else
      {
         src = cs + 8 * (ch-' ');
         cd1 = *(unsigned int*)(src+0);
         cd2 = *(unsigned int*)(src+4);
      }
      
      for(pixel = 0; pixel < 8; pixel++)
      {
         line = 1 << (7-pixel);
         
         if( (cd1 >>  0) & line )
         {
            vd2  = fc << 24;
         }
         else
         {
            vd2  = bc << 24;
         }
         
         if( (cd1 >>  8) & line )
         {
            vd2 |= fc << 16;
         }
         else
         {
            vd2 |= bc << 16;
         }
         
         if( (cd1 >> 16) & line )
         {
            vd2 |= fc <<  8;
         }
         else
         {
            vd2 |= bc <<  8;
         }
   
         if( (cd1 >> 24) & line )
         {
            vd2 |= fc <<  0;
         }
         else
         {
            vd2 |= bc <<  0;
         }
         
         if( (cd2 >>  0) & line )
         {
            vd1  = fc << 24;
         }
         else
         {
            vd1  = bc << 24;
         }
         
         if( (cd2 >>  8) & line )
         {
            vd1 |= fc << 16;
         }
         else
         {
            vd1 |= bc << 16;
         }
         
         if( (cd2 >> 16) & line )
         {
            vd1 |= fc <<  8;
         }
         else
         {
            vd1 |= bc <<  8;
         }
         
         if( (cd2 >> 24) & line )
         {
            vd1 |= fc <<  0;
         }
         else
         {
            vd1 |= bc <<  0;
         }
         
         vram = (unsigned int*)&ds->o_buffer[((x++)*240) + 236 - y];
         *vram = vd2;
         vram--;
         *vram = vd1;
         if(x >= 320) return;
      }
   }
}



void SOMFontDrawString(GPDRAWSURFACE *ds, int x, int y, 
   const unsigned char *cs, const unsigned char *string,
   unsigned char fc, unsigned char bc, unsigned char tc)
{
   register unsigned char ch;

   if((y < -7) || (y > 239)) return;

   while(*string)
   {
      ch = *(string++);
      
      if(*cs < 7)
      {
         if((ch < 0x20) || (ch > 0x7f)) ch=' ';
         if((*cs ==  2) && (ch > 0x5f)) ch-=0x20;
      }
      
      SOMFontDrawChar(ds, x, y, cs, ch, fc, bc, tc);
      x+=8;
   }
}



void SOMFontDrawOutlineString(GPDRAWSURFACE *ds, int x, int y, 
   const unsigned char *cs, const unsigned char *string,
   unsigned char fc, unsigned char oc)
{
   const char *s;
   unsigned char tc;
   
   tc = SOMFontFindNonTransColor(fc, oc);

   if((y < -7) || (y > 239)) return;

   for(s=string; *s; s++)
   {
      SOMFontDrawChar(ds, x, y-1, cs, *s, oc, tc, tc);
      SOMFontDrawChar(ds, x-1, y, cs, *s, oc, tc, tc);
      SOMFontDrawChar(ds, x+1, y, cs, *s, oc, tc, tc);
      SOMFontDrawChar(ds, x, y+1, cs, *s, oc, tc, tc);
      SOMFontDrawChar(ds, x, y,   cs, *s, fc, tc, tc);
      x+=8;
   }
}
