
#include "os.h"

#ifdef GP32_SDK
#include <stdio.h>
#include <gpstdio.h>
#include <gpmem.h>

void *gm_mycalloc(size_t num_elem, size_t size_elem)
{
	char *new_ptr;
	new_ptr=gm_calloc(num_elem,size_elem);
	gm_memset(new_ptr,0,num_elem*size_elem);
	return (void*)new_ptr;	
}


void *gm_realloc(void *ptr, size_t size)
{
	char *new_ptr;
	
	if (size)
	{
		new_ptr=(char*)gm_malloc(size);
		if (ptr)
		{
			gm_memcpy(new_ptr,(char*)ptr,size);
			gm_free(ptr);
		}
		return (void*)new_ptr;
	}
	else
	{
		if (ptr) gm_free(ptr);
		return (void*)NULL;
	}
}


size_t gp_callback_read(void *ptr, size_t size, size_t nmemb, void *datasource)
{
   F_HANDLE fh = (F_HANDLE)datasource;
   long bytesread;
   long err;
   
   if(!(size*nmemb))
   {
      return 0;
   }
   err = GpFileRead( fh, ptr, size*nmemb, &bytesread );

   return bytesread;
}


int gp_callback_seek(void *datasource, ogg_int64_t offset, int whence)
{
   F_HANDLE fh = (F_HANDLE)datasource;
   long position;
   long seek_mode;
   
   switch(whence)
   {
      case SEEK_CUR:
         seek_mode = FROM_CURRENT;
         break;
      case SEEK_SET:
         seek_mode = FROM_BEGIN;
         break;
      case SEEK_END:
         seek_mode = FROM_END;
         break;
      default:
         return -1;
   }
   
   if( GpFileSeek( fh, seek_mode, (ulong)offset, &position ) == SM_OK )
   {
      return 0;
   }
   else
   {
      // set by-thread errno
      return -1;
   }
}


int gp_callback_close(void *datasource)
{
   F_HANDLE fh = (F_HANDLE)datasource;
   ERR_CODE err;
   
   err = GpFileClose( fh );
   
   if( err == SM_OK )
   {
      return 0;
   }
   else
   {
      // set by-thread errno
      return -1;
   }
}


long gp_callback_tell(void *datasource)
{
   F_HANDLE fh = (F_HANDLE)datasource;
   long position;
   
   if(GpFileSeek( fh, FROM_CURRENT, 0, &position ) != SM_OK )
   {
      position = -1;
   }
   return position;
}
#endif
