/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis 'TREMOR' CODEC SOURCE CODE.   *
 *                                                                  *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis 'TREMOR' SOURCE CODE IS (C) COPYRIGHT 1994-2002    *
 * BY THE Xiph.Org FOUNDATION http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: #ifdef jail to whip a few platforms into the UNIX ideal.

 ********************************************************************/
#ifndef _OS_TYPES_H
#define _OS_TYPES_H

#ifdef _LOW_ACCURACY_
#  define X(n) (((((n)>>22)+1)>>1) - ((((n)>>22)+1)>>9))
#  define LOOKUP_T const unsigned char
#else
#  define X(n) (n)
#  define LOOKUP_T const ogg_int32_t
#endif

typedef signed short ogg_int16_t;
typedef signed int ogg_int32_t;
typedef unsigned int ogg_uint32_t;
typedef signed long long ogg_int64_t;   

#define _ogg_static  static
#define _ogg_inline  inline

/* some max values for statified arrays */
#define MAX_CODEENTRIES 16384 /* sharedbook.c (0 use malloc())*/
#define MAX_CHANNELS 2 /* mapping0.c */

#ifdef POSIX
#define ov_callback_read   fread
#define ov_callback_seek   fseek
#define ov_callback_close  fclose
#define ov_callback_tell   ftell
#else
#ifdef mirkoSDK
#include <gp32.h>
#include <cpp_prototypes.h>

/* make it easy on the folks that want to compile the libs with a
   different malloc than stdlib */
#define _ogg_malloc        malloc
#define _ogg_calloc        calloc
#define _ogg_realloc       realloc
#define _ogg_free          free
#define _ogg_memset        memset

#define ov_callback_read   smc_fread
#define ov_callback_seek   smc_fseek
#define ov_callback_close  smc_fclose
#define ov_callback_tell   smc_ftell

#else
#ifdef GP32_SDK
#include <gpdef.h>
#include <gpstdio.h>
#include <gpmem.h>

/* make it easy on the folks that want to compile the libs with a
   different malloc than stdlib */
#define _ogg_malloc        gm_malloc
#define _ogg_calloc        gm_mycalloc
#define _ogg_realloc       gm_realloc
#define _ogg_free          gm_free
#define _ogg_memset        gm_memset

void *gm_mycalloc(size_t num_elem, size_t size_elem);
void *gm_realloc(void *ptr, size_t size);

size_t gp_callback_read(void *ptr, size_t size, size_t nmemb, void *datasource);
int gp_callback_seek(void *datasource, ogg_int64_t offset, int whence);
int gp_callback_close(void *datasource);
long gp_callback_tell(void *datasource);

#define ov_callback_read   gp_callback_read
#define ov_callback_seek   gp_callback_seek
#define ov_callback_close  gp_callback_close
#define ov_callback_tell   gp_callback_tell

#else
#error SDK not defined
#endif /* GP32_SDK */
#endif /* mirkoSDK */
#endif /* POSIX */

/* no forward declaration elsewhere */
int _ilog(unsigned int v);

#endif  /* _OS_TYPES_H */
