
#include <global.h>

const char *config_file_name   = "gp:\\GPETC\\" PROJECT_NAME_SHORT ".INI";

/* defaults */
char *config_file_playlist     = "gp:\\OGG\\PLAYLIST.BIN";
char *config_ogg_dir           = "gp:\\OGG";
int  config_color_background   = 0xff;
int  config_color_text         = 0x02;
int  config_color_highlight    = 0x97;
int  config_color_outline      = 0x97;
int  config_color_lines        = 0x07;
int  config_color_nontrans     = 0x00;
int  config_position_highlight = 8;
int  config_voltage_adc_port   = -1;
char *config_version           = "Builtin Config";
char *config_info_line1        = "ARTIST=";
char *config_info_line2        = "TITLE=";
char *config_charset           = &SOMFont_default_charset[0];
int  config_cpu_master_speed   = 136000000;
int  config_cpu_div_factor     = 0x3c011;
int  config_cpu_clk_mode       = 3;
int  config_wind_step          = 10000;
int  config_random_mode        = RANDOMPLAY;
char config_playlist_end_marker= '=';
int  config_write_delay        = 10000;
int  config_key[NUMBER_FUNCTIONS] =
{
/* FUNCTION_PREVIOUS_TRACK */
   GPC_VK_UP,
/* FUNCTION_NEXT_TRACK */
   GPC_VK_DOWN,
/* FUNCTION_REWIND */
   GPC_VK_FL,
/* FUNCTION_FAST_FORWARD */
   GPC_VK_FR,
/* FUNCTION_SEPARATOR_LEFT */
   GPC_VK_LEFT,
/* FUNCTION_SEPARATOR_RIGHT */
   GPC_VK_RIGHT,
/* FUNCTION_MOVE_TRACK_UP */
   GPC_VK_SELECT | GPC_VK_UP,
/* FUNCTION_MOVE_TRACK_DOWN */
   GPC_VK_SELECT | GPC_VK_DOWN,
/* FUNCTION_RANDOM */
   GPC_VK_START | GPC_VK_FA,
/* FUNCTION_PAUSE */
   GPC_VK_START | GPC_VK_FB,
/* FUNCTION_LOCK */
   GPC_VK_START | GPC_VK_SELECT | FUNCTION_OVERRIDE_LOCK,
/* FUNCTION_HELP_SCREEN */
   GPC_VK_FA | FUNCTION_OVERRIDE_LOCK,
/* FUNCTION_INFO_SCREEN */
   GPC_VK_FB,
};
int  config_key_repeat_delay1       = 1000;
int  config_key_repeat_delay2       = 400;
int  config_key_repeat_delay3       = 50;
int  config_key_repeat_threshold12  = 2;
int  config_key_repeat_threshold23  = 7;
struct status_t status;

#define CONFIG_VERSION_SIZE   19


int config_binary8(const char *value)
{
   char *p = (char*) value;
   int retval = 0, i;
   
   for(i = 7; i >= 0; i--)
   {
      switch(*p)
      {
         case ' ':
            i++;
         case '0':
            p++;
            break;
         case '1':
            retval |= 1 << i;
            p++;
            break;
         default:
            return -1;
      }
   }
   
   return retval;
}



void config_set_key_value( const char *key, const char *value )
{
   int ivalue;
   
   if(!strcasecmp(key,"file.playlist"))
   {
      config_file_playlist = strdup( value );
      return;
   }
   if(!strcasecmp(key,"file.charset"))
   {
      config_charset = SOMFontLoad( value );
      if( !config_charset ) config_charset = &SOMFont_default_charset[0];
sprintf(conbuf,"font addr:%08x(%08x) type: %d\n",
      config_charset,&SOMFont_default_charset[0],*config_charset);
SOMFontConsolePrint(conbuf);
      return;
   }
   if(!strcasecmp(key,"file.ogg.dir"))
   {
      config_ogg_dir = strdup( value );
      return;
   }
   
   if(!strcasecmp(key,"color.background"))
   {
      ivalue = config_binary8( value );
      if(ivalue >= 0) config_color_background = ivalue;
      return;
   }
   if(!strcasecmp(key,"color.text"))
   {
      ivalue = config_binary8( value );
      if(ivalue >= 0) config_color_text = ivalue;
      return;
   }
   if(!strcasecmp(key,"color.highlight"))
   {
      ivalue = config_binary8( value );
      if(ivalue >= 0) config_color_highlight = ivalue;
      return;
   }
   if(!strcasecmp(key,"color.outline"))
   {
      ivalue = config_binary8( value );
      if(ivalue >= 0) config_color_outline = ivalue;
      return;
   }
   if(!strcasecmp(key,"color.lines"))
   {
      ivalue = config_binary8( value );
      if(ivalue >= 0) config_color_lines = ivalue;
      return;
   }
   
   if(!strcasecmp(key,"position.highlight"))
   {
      ivalue = strtol( value, NULL, 0 );
      if( (ivalue >= 0) && (ivalue <= 19) ) config_position_highlight = ivalue;
      return;
   }
   
   if(!strcasecmp(key,"version"))
   {
      config_version             = strdup( value );
      if(strlen(config_version) > CONFIG_VERSION_SIZE)
      {
         /* truncate if too long */
         config_version[CONFIG_VERSION_SIZE] = '\0';
      }
      return;
   }
   if(!strcasecmp(key,"comment.info1"))
   {
      config_info_line1          = strdup( value );
      return;
   }
   if(!strcasecmp(key,"comment.info2"))
   {
      config_info_line2          = strdup( value );
      return;
   }

   if(!strcasecmp(key,"voltage.detection"))
   {
      char *port = "?\n";
      config_voltage_adc_port    = strtol( value, NULL, 0);
      if( config_voltage_adc_port > 7 ) config_voltage_adc_port = -1;
      if( config_voltage_adc_port >= 0 )
      {
         *port = '0' + config_voltage_adc_port;
         SOMFontConsolePrint( "Activating voltage detection mod support" /*wrap*/
                              "on ADC port " );
         SOMFontConsolePrint( &port[0] );
      }
      return;
   }
   
   if(!strcasecmp(key,"cpu.mclk"))
   {
      config_cpu_master_speed    = strtol( value, NULL, 0 );
      switch( config_cpu_master_speed )
      {
         case 136:
            config_cpu_div_factor=0x3c011;
            config_cpu_master_speed=136000000;
            config_cpu_clk_mode=3;
            break;
         case 114:
            config_cpu_div_factor=0x44021;
            config_cpu_master_speed=114000000;
            config_cpu_clk_mode=2;
            break;
      }
      return;
   }
   if(!strcasecmp(key,"cpu.divfactor"))
   {
      config_cpu_div_factor      = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"cpu.clkmode"))
   {
      config_cpu_clk_mode        = strtol( value, NULL, 0 );
      return;
   }
   
   if(!strcasecmp(key,"playlist.end.marker"))
   {
      ivalue = strtol( value, NULL, 0 );
      if((ivalue == 0) || ((ivalue >= 0x20) && (ivalue <= 0x7f)))
      {
         config_playlist_end_marker = ivalue;
         return;
      }
      if(ivalue != 1)
      {
         SOMFontConsolePrint( "Wrong playlist end marker \"" );
         SOMFontConsolePrint( value );
         SOMFontConsolePrint( "\"\n" );
      }
      return;
   }
   if(!strcasecmp(key,"wind.step"))
   {
      ivalue = strtol( value, NULL, 0 );
      if(ivalue < 0) ivalue =- ivalue;
      if(ivalue > 300)
      {
         config_wind_step = ivalue;
      }
      else
      {
         config_wind_step = ivalue * 1000;
      }
      return;
   }
   if(!strcasecmp(key,"write.delay"))
   {
      ivalue = strtol( value, NULL, 0 );
      if((ivalue > 0) && (ivalue <= 120))
      {
         config_write_delay = ivalue * 1000;
      }
      return;
   }
   if(!strcasecmp(key,"random.mode"))
   {
      if(!strncasecmp(value,"random",6))
      {
         config_random_mode = RANDOMPLAY;
         return;
      }
      if(!strcasecmp(value,"shuffle"))
      {
         config_random_mode = SHUFFLE;
         return;
      }
      SOMFontConsolePrint( "Randommode \"" );
      SOMFontConsolePrint( value );
      SOMFontConsolePrint( "\" unknown\n" );
      return;
   }
   
   
   if(!strcasecmp(key,"key.previous.track"))
   {
      config_key[FUNCTION_PREVIOUS_TRACK] = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.next.track"))
   {
      config_key[FUNCTION_NEXT_TRACK]     = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.rewind"))
   {
      config_key[FUNCTION_REWIND]         = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.fast.forward"))
   {
      config_key[FUNCTION_FAST_FORWARD]   = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.separator.left"))
   {
      config_key[FUNCTION_SEPARATOR_LEFT] = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.separator.right"))
   {
      config_key[FUNCTION_SEPARATOR_RIGHT]= strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.move.track.up"))
   {
      config_key[FUNCTION_MOVE_TRACK_UP]  = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.move.track.down"))
   {
      config_key[FUNCTION_MOVE_TRACK_DOWN]= strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.random"))
   {
      config_key[FUNCTION_RANDOM]         = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.pause"))
   {
      config_key[FUNCTION_PAUSE]          = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.lock"))
   {
      config_key[FUNCTION_LOCK]           = strtol( value, NULL, 0 )
                                          | FUNCTION_OVERRIDE_LOCK ;
      return;
   }
   if(!strcasecmp(key,"key.help.screen"))
   {
      config_key[FUNCTION_HELP_SCREEN]    = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.info.screen"))
   {
      config_key[FUNCTION_INFO_SCREEN]    = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.repeat.delay1"))
   {
      config_key_repeat_delay1            = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.repeat.delay2"))
   {
      config_key_repeat_delay2            = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.repeat.delay3"))
   {
      config_key_repeat_delay3            = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.repeat.threshold12"))
   {
      config_key_repeat_threshold12       = strtol( value, NULL, 0 );
      return;
   }
   if(!strcasecmp(key,"key.repeat.threshold23"))
   {
      config_key_repeat_threshold23       = strtol( value, NULL, 0 );
      return;
   }

   SOMFontConsolePrint( "Parameter \"" );
   SOMFontConsolePrint( key );
   SOMFontConsolePrint( "\" unknown\n" );
}



void config_parse( char *buffer, int size )
{
   int phase;
   char *key, *value, *c;
   
   key=buffer;
   value=buffer;
   
   for(phase=1,c=buffer;(c-buffer) < size;c++)
   {
      switch(phase)
      {
         case 1:
            /* skip spaces and tabs */
            if((*c == ' ') || (*c == '\t')) key=c+1;
            else phase=2;
            break;
         case 2:
            /* convert keyname to lower case */
            *c=tolower(*c);
            /* look for end of key */
            if((*c == ' ') || (*c == '\t'))
            {
               *c = '\0';
               phase=3;
            }
            /* fall through */
         case 3:
            /* look for '=' as delimiter beween key and value */
            if(*c == '=')
            {
               *c = '\0';
               value=c+1;
               phase=4;
            }
            break;
         case 4:
            /* skip spaces and tabs at the start of value */
            if((*c == ' ') || (*c == '\t')) value=c+1;
            else phase=5;
            break;
         case 5:
            /* look for end of key */
            if((*c == '\n') || (*c == '\r'))
            {
               *c = '\0';
               phase=1;
               if((*key != '#') && (*key != ';')) config_set_key_value(key,value);
               key=c+1;
            }
         default:
            break;
      }
      // search for beginning of line;
      if(*c == '\n')
      {
         key=c+1;
         phase=1;
      }
   }
}



void config_read()
{
   F_HANDLE fh;
   char *data, *p;
   ulong data_size, rest, read;
   ERR_CODE err;
   const char *loaded = " Loaded";
   
   /* set status defaults */
   status.index       =  0;
   status.field1_size = 15;
   status.playmode    =  0;

   sprintf(conbuf, "config file: %s\n", config_file_name );
   SOMFontConsolePrint( conbuf );

   /* get size of file */
   err = GpFileGetSize( config_file_name, &data_size );

   /* file could not be read */
   if(err != SM_OK) return;
   
   sprintf(conbuf, "found (%d bytes)\n", data_size );
   SOMFontConsolePrint( conbuf );
   
   if(data_size > 102400)
   {
      SOMFontConsolePrint( "config file > 100k, probably corrupt, not loaded." );
      return;
   }
   
   /* get memory for file */
   data = (char*)malloc(data_size);
   if(!data) return;
   
   /* mark config as loaded */
   memcpy( config_version, loaded, strlen(loaded) );
   
   /* read file to memory */
   err = GpFileOpen( config_file_name, OPEN_R, &fh );
   p = data;
   rest = data_size;
   
   while(rest)
   {
      err = GpFileRead( fh, p, rest, &read );
      p += read;
      rest -= read;
   }
   
   GpFileClose( fh );
   
   /* analyse file */
   config_parse( data, data_size );
   
   /* clean up */
   free(data);
   if( config_random_mode != RANDOMPLAY ) 
   {
      status.playmode &= (~PLAYMODE_RANDOM);
   }
   config_color_nontrans = SOMFontFindNonTransColor(config_color_text,
                                                    config_color_background);
}
