
#include <gpstdio.h>
#include <gpstdlib.h>
#include <gpgraphic.h>
#include <gpmm.h>
#include <gpmem.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <8x8font.h>
#include <ivorbisfile.h>


#define PROJECT_NAME_SHORT "SVORBIS"
#define PROJECT_NAME_LONG  "SvOrbis"
#define PROJECT_VERSION    "0.9e"
#define COLOR_CONSOLE_BACKGROUND 0xe0
#define COLOR_CONSOLE_TEXT       0xff
#define COLOR_CONSOLE_LINE       0x00

struct entry_t
{
   struct entry_t *prev;
   struct entry_t *next;
   char *filename;
   char *artist;
   char *title;
   long size;
   long id;
};

struct status_t
{
   int index;
   int field1_size;
   int playmode;
};

#define PLAYMODE_RANDOM 0x01
#define PLAYMODE_LOCKED 0x02

enum PLAYMODE
{
   stop = 0,
   play,
   pause,
   seek,
   next
};

enum RANDOMMODE
{
   RANDOMPLAY,
   SHUFFLE
};



/* gpmain.c */
/* global vars */
extern char conbuf[80];
extern GPDRAWSURFACE  gpDraws[4], *gpDraw;
extern GP_HPALETTE    gpPalette;
/* prototypes */
void KeyTest();
void KeyString( char *keystring, int key, int length );
void Delay( unsigned int millisec );
void Time( char *d, long time, int withhour );
void DrawOnceOnly( GPDRAWSURFACE *draw, int titleonly );
void DrawOncePerTrack( GPDRAWSURFACE *draw, struct entry_t *entry, int freq );
void PlaylistEntry( GPDRAWSURFACE *draw, struct entry_t *entry,
                    int y, char fg, char bg );
void Playlist( GPDRAWSURFACE *draw, struct entry_t *actual );
void MainLoop();
void DrawHelpScreen( GPDRAWSURFACE *draw );
void Setup();
void GpMain( void *arg );



/* playlist.c */
/* global vars */
extern long playlist_total_size;
extern long playlist_total_num;
extern struct entry_t *playlist_first, *playlist_last;
/* prototypes */
char *my_strdup(const char *src);
char *my_strndup(const char *src, int maxsize);
struct entry_t *playlist_getprev( struct entry_t* entry );
struct entry_t *playlist_getnext( struct entry_t* entry );   
struct entry_t *playlist_append( char *filename,
                                 char *artist, char *title, long size );
void playlist_delete( struct entry_t *entry );
void playlist_remove( struct entry_t *entry );
void playlist_insert( struct entry_t *entry, struct entry_t *newone );
void playlist_swap( struct entry_t *entry, int direction );
void playlist_recalc();
void playlist_shuffle();
int playlist_getindex( struct entry_t *entry );
struct entry_t *playlist_getentry( int num );



/* config.c */
#define FUNCTION_OVERRIDE_LOCK   0x1000
#define FUNCTION_PREVIOUS_TRACK   0
#define FUNCTION_NEXT_TRACK       1
#define FUNCTION_REWIND           2
#define FUNCTION_FAST_FORWARD     3
#define FUNCTION_SEPARATOR_LEFT   4
#define FUNCTION_SEPARATOR_RIGHT  5
#define FUNCTION_MOVE_TRACK_UP    6
#define FUNCTION_MOVE_TRACK_DOWN  7
#define FUNCTION_RANDOM           8
#define FUNCTION_PAUSE            9
#define FUNCTION_LOCK            10
#define FUNCTION_HELP_SCREEN     11
#define FUNCTION_INFO_SCREEN     12
#define NUMBER_FUNCTIONS         13
/* global vars */
extern char *config_file_playlist;
extern char *config_ogg_dir;
extern int  config_color_background;
extern int  config_color_highlight;
extern int  config_color_text;
extern int  config_color_outline;
extern int  config_color_lines;
extern int  config_color_nontrans;
extern int  config_position_highlight;
extern char *config_version;
extern char *config_info_line1;
extern char *config_info_line2;
extern char *config_charset;
extern int  config_voltage_adc_port;
extern int  config_cpu_master_speed;
extern int  config_cpu_div_factor;
extern int  config_cpu_clk_mode;
extern int  config_wind_step;
extern int  config_random_mode;
extern char config_playlist_end_marker;
extern int  config_write_delay;
extern int  config_key[NUMBER_FUNCTIONS];
extern int  config_key_repeat_delay1;
extern int  config_key_repeat_delay2;
extern int  config_key_repeat_delay3;
extern int  config_key_repeat_threshold12;
extern int  config_key_repeat_threshold23;
extern struct status_t status;
/* prototypes */
void config_read();



/* logo.c */
/* global vars */
/* prototypes */
void logo_draw( GPDRAWSURFACE *draw );



/* vorbis.c */
/* global vars */
extern int vorbis_rate;
extern int vorbis_channels;
extern long vorbis_decoded_size;
extern OggVorbis_File vorbis_vf;
/* prototypes */
int vorbis_init();
int vorbis_seek( int offset );
int vorbis_scan( const char *filename, struct entry_t *entry );
static inline long snddrv_decode( char *buffer, int length )
{
   return ov_read( &vorbis_vf, buffer, length, NULL );
}
static inline long vorbis_tell_current()
{
   return (long)ov_time_tell(&vorbis_vf);
}
static inline int vorbis_close()
{
   return ov_clear( &vorbis_vf );
}

/* gpvorbis.c */
int gpvorbis_open( const char *filename );



/* gpstorage.c */
/* global vars */
/* prototypes */
int gpstorage_init();
char *gpstorage_buffer_read( char *filename, ulong *buffersize );
void gpstorage_buffer_write( char *filename, char *buffer, ulong buffersize );

/* storage.c */
/* global vars */
/* prototypes */
struct entry_t* storage_playlist_entry(int i);
int storage_playlist_size();
void storage_status_read();
void storage_status_write();
void storage_playlist_read();
void storage_playlist_write();
struct entry_t *storage_inplaylist( char *filename );



/* gpsnddrv.c */
/* global vars */
extern volatile enum PLAYMODE  playmode;
extern volatile int snddrv_stopped;
/* prototypes */
void snddrv_cpuspeed( int master_speed, int div_factor, int clk_mode );
int  snddrv_resync( int freq );
int  snddrv_init();



/* some defines for more straightforward coding */
/* must be done after including all (system-) prototypes */

#define malloc    gm_malloc
#define free      gm_free
#define memset    gm_memset
#define strdup    my_strdup
#define strndup   my_strndup
#define strcpy    gm_strcpy
#define strcat    gm_strcat
#define strlen    gm_lstrlen
#define sprintf   gm_sprintf

