
#include <global.h>

/* ADC from gp32.h */
#define rADCCON         (*(volatile unsigned *)0x15800000)                      
#define rADCDAT         (*(volatile unsigned *)0x15800004)                      


char  conbuf[80];

GPDRAWSURFACE  gpDraws[4], *gpDraw;
GP_HPALETTE    gpPalette;

extern unsigned char SOMFont_default_charset[];
const int playlist_y = 60;



void Delay( unsigned int millisec )
{
   unsigned int nTick;
   nTick=GpTickCountGet();
   while( (GpTickCountGet())<millisec+nTick );
}



void GpWaitKeyA()
{
   int i;
   do
   {
      GpKeyGetEx(&i);
   } while(!(i & GPC_VK_FA));
   do
   {
      GpKeyGetEx(&i);
   } while( (i & GPC_VK_FA));
}



void Time( char *d, long time, int withhour )
{
   int   hours = 0;
   int   mins  = 0;
   int   secs  = 0;
   
   time /= 1000;
   if( withhour )
   {
      if( time >= 360000 ) /* >= 100 hours won't fit */
      {
         strcpy(d,"**:**:**");
         return;
      }
      
      hours = time / 3600;
      mins  = (time / 60) % 60;
      secs  = time % 60;
      
      if( (hours / 10) > 0 )
      {
         *(d++) = '0' + hours / 10;
      }
      else
      {
         *(d++) = ' ';
      }
      *(d++) = '0' + hours % 10;
      *(d++) = ':';
      *(d++) = '0' + mins / 10;
   }
   else
   {
      if( time >= 6000 ) /* >= 100 minutes won't fit */
      {
         strcpy(d,"**:**");
         return;
      }
      
      mins = time / 60;
      secs = time % 60;
      
      if( (mins / 10) > 0 )
      {
         *(d++) = '0' + mins / 10;
      }
      else
      {
         *(d++) = ' ';
      }
   }
   *(d++) = '0' + mins % 10;
   *(d++) = ':';
   *(d++) = '0' + secs / 10;
   *(d++) = '0' + secs % 10;
}



void PlaylistEntry( GPDRAWSURFACE *draw, struct entry_t *entry,
                     int y, char fg, char bg )
{
   char  text[41], *s, *d = text;
   int   pos = 0, title_end = 35;
   unsigned int *vram, color;
   
   color = (config_color_lines << 24) | (config_color_lines << 16) |
           (config_color_lines <<  8) | (config_color_lines);
   
   if(entry)
   {
      s = entry->artist;
      if(!s) s = entry->filename+4;
      for(;(pos<status.field1_size)&&*s;pos++)
      {
         *(d++) = *(s++);
      }
      for(;pos<status.field1_size;pos++)
      {
         *(d++) = ' ';
      }
      
      if(entry->title)
      {
         for(s=entry->title;(pos<title_end)&&*s;pos++)
         {
            *(d++) = *(s++);
         }
      }
      for(;pos<title_end;pos++)
      {
         *(d++) = ' ';
      }
      Time( d, entry->size, 0 );
   }
   else
   {
      for(;pos<40;pos++)
      {
         *(d++) = config_playlist_end_marker;
      }
   }
   
   SOMFontFastString( draw, 0, y, config_charset, text, fg, bg );
   
   vram = (unsigned int*)&draw->o_buffer[((status.field1_size*8-1)*240) + 236 - y];
   *(vram--) = color;
   *(vram)   = color;
   vram = (unsigned int*)&draw->o_buffer[((title_end*8-1)*240) + 236 - y];
   *(vram--) = color;
   *(vram)   = color;
}



void Playlist( GPDRAWSURFACE *draw, struct entry_t *actual )
{
   struct entry_t *entry;
   const int mainline = config_position_highlight;
   int i;

   /* print actual */
   PlaylistEntry( draw, actual, playlist_y+8*mainline,
                  config_color_text, config_color_highlight );
   
   entry = actual;
   for(i = mainline-1; i >= 0; i--)
   {
      if(entry)
      {
         entry = playlist_getprev( entry );
         if( !entry && !config_playlist_end_marker )
         {
            entry = playlist_last;
         }
      }
      else
      {
         entry = playlist_last;
      }
      PlaylistEntry( draw, entry, playlist_y+8*i,
                     config_color_text, config_color_background );
   }
   
   entry = actual;
   for(i = mainline+1; i < 20; i++)
   {
      if(entry)
      {
         entry = playlist_getnext( entry );
         if( !entry && !config_playlist_end_marker )
         {
            entry = playlist_first;
         }
      }
      else
      {
         entry = playlist_first;
      }
      PlaylistEntry( draw, entry, playlist_y+8*i,
                     config_color_text, config_color_background );
   }
}



void DrawOnceOnly( GPDRAWSURFACE *draw, int titleonly )
{
   const char *version  = "Version " PROJECT_VERSION;
   char       helpinfo[24];
   logo_draw( draw );
   
   SOMFontDrawOutlineString( draw, 318 - 8 * strlen( version ), 11,
                             config_charset,
                             version,
                             config_color_text,
                             config_color_outline );
   
   SOMFontDrawOutlineString( draw, 318 - 8 * strlen( config_version ), 22,
                             config_charset,
                             config_version,
                             config_color_text,
                             config_color_outline );
   
   if( !titleonly )
   {
      GpLineDraw( draw,   0, playlist_y-  2, 319, playlist_y-  2, config_color_lines );
      GpLineDraw( draw,   0, playlist_y+161, 319, playlist_y+161, config_color_lines );
      GpLineDraw( draw, 279,      playlist_y-  2, 279, playlist_y+161, config_color_lines );
      strcpy(helpinfo,"Hold ");
      KeyString(helpinfo,config_key[FUNCTION_HELP_SCREEN],23-9);
      strncat(helpinfo," for help",23);
      SOMFontFastString( draw, 160-strlen(helpinfo)*4, 232, config_charset, helpinfo,
                         config_color_text, config_color_background );
   }
}



void DrawStatusLine( GPDRAWSURFACE *draw, int mode )
{
   /*
    * mode: 
    * 0 with voltage 
    * 1 without voltage
    * 2 without voltage : writing Playlist info
    */
   
   volatile int voltage;   /* 0123456789012345678901234567890123456789 */
                           /* v.vvV      Random Lock ttt Trks|hh:mm:ss */  
   char statusline[41] = "                           Trks          ";
   const char random[] = "Random";
   const char lock[]   =        "Lock";
   const char write[]  = "*SMC WRITE*";
   const char pausem[] = "<> Pause <>";
   char *d;
   
   if(( config_voltage_adc_port >= 0 ) && !mode )
   {
      unsigned short adccon;
      
      adccon = 0x6001; /* prescaler value is 128 */
      adccon += ((unsigned short)config_voltage_adc_port)<<3;
      rADCCON = adccon;
   }
   
   /* number of tracks */
   d = &statusline[23];
   if( playlist_total_num / 100 )
   {
      *(d++) = '0' + playlist_total_num / 100;
   }
   else
   {
      *(d++) = ' ';
   }
   if( playlist_total_num / 10 )
   {
      *(d++) = '0' + (playlist_total_num / 10) % 10;
   }
   else
   {
      *(d++) = ' ';
   }
   *(d++) = '0' + playlist_total_num % 10;
   
   if( mode == 2 )
   {
      /* smc write message */
      memcpy(&statusline[11], &write[0], 11 );
   }
   else
   {
      if( playmode == pause )
      {
         /* pause message */
         memcpy(&statusline[11], &pausem[0], 11 );
      }
      else
      {
         /* random? */
         if( status.playmode & PLAYMODE_RANDOM )
         {
            memcpy( &statusline[11], &random[0], 6 );
         }
         /* lock? */
         if( status.playmode & PLAYMODE_LOCKED )
         {
            memcpy( &statusline[18], &lock[0], 4 );
         }
      }
   }
   
   /* total time */
   Time( &statusline[32], playlist_total_size, 1 );
   
   if(( config_voltage_adc_port >= 0 ) && !mode )
   {
      /* read voltage */
      while (rADCCON & 1); /* wait for conversion to end */
      voltage = (((int)rADCDAT) * 330) >> 10;
      d = &statusline[0];
      *(d++) = '0' + voltage / 100;
      *(d++) = '.';
      *(d++) = '0' + (voltage / 10) % 10;
      *(d++) = '0' + voltage % 10;
      *(d)   = 'V'; /* *(d) = '0' + config_voltage_adc_port; */
   }

   if( mode )
   {
      SOMFontFastString( draw, 40, playlist_y+164,
                         config_charset,
                         &statusline[5],
                         config_color_text,
                         config_color_background );
   }
   else
   {
      SOMFontFastString( draw,  0, playlist_y+164,
                         config_charset,
                         &statusline[0],
                         config_color_text,
                         config_color_background );
   }
}



void DrawOncePerTrack( GPDRAWSURFACE *draw, struct entry_t *entry, int freq )
{
   /*
    * mode:
    * 0: just filename, artist and title
    * 1: full info
    */
   
   int y = 32;
   char timebuf[12];
   
   /* clear out actual ogg info */
   GpRectFill( NULL, draw, 0, y, gpDraw->buf_w, 24, config_color_background );

   if( freq )
   {
      strcpy(timebuf," 0:00/**:**");
      Time(&timebuf[6], entry->size, 0);
      
      SOMFontFastString( draw, 232, y,
                         config_charset,
                         timebuf,
                         config_color_text,
                         config_color_background );
   }
   SOMFontFastString( draw,   0, y,
                      config_charset,
                      entry->filename+4,
                      config_color_text,
                      config_color_background );

   SOMFontFastString( draw,   0, y + 8,
                      config_charset,
                      entry->artist,
                      config_color_text,
                      config_color_background );

   SOMFontFastString( draw,   0, y + 16,
                      config_charset,
                      entry->title,
                      config_color_text,
                      config_color_background );

   if( freq )
   {
      DrawStatusLine( draw, 0 );
      sprintf( timebuf, "OF:%d", freq );
      SOMFontFastString( draw, 256, 232, config_charset, timebuf,
         config_color_text, config_color_background);
   }
}



void MainLoop()
{
   int skip = 0, dirty = 0, redraw = 0, key = 0, lastkey = 0,
       function = 0, waitrelease = 0;
   unsigned int nextupdate = 0xffffffff, repeatcount = 0;
   struct entry_t *entry = playlist_first;
   GPDRAWSURFACE *gpShow = gpDraw;
   
   
   char timebuf[8];
   gm_memset(timebuf, 0, 8);

   entry    = playlist_getentry( status.index );
   redraw   = 1;
   playmode = next;
   
   for(;;)
   {
      if( (gpDraw == gpShow) && redraw )
      {
         if( gpDraw == &gpDraws[0] )
         {
            gpDraw = &gpDraws[1];
         }
         else
         {
            gpDraw = &gpDraws[0];
         }
         
         Playlist( gpDraw, entry );
         DrawStatusLine( gpDraw, 1 );
         GpSurfaceFlip( gpDraw );
         gpShow = gpDraw;
         redraw = 0;
      }
      
      if( !skip )
      {
         if( dirty )
         {
            nextupdate = GpTickCountGet() + config_write_delay;
            dirty = 0;
         }
      
         if( GpTickCountGet() > nextupdate )
         {
            status.index = playlist_getindex( entry );
            DrawStatusLine( gpDraw, 2 );
            storage_playlist_write();
            DrawStatusLine( gpDraw, 1 );
            nextupdate = 0xffffffff;
         }
         
         if(playmode == next)
         {
            int freq;
            gpvorbis_open( entry->filename );
            entry->size = vorbis_decoded_size;
            freq = snddrv_resync( vorbis_rate );
            DrawOncePerTrack( &gpDraws[0], entry, freq );
            DrawOncePerTrack( &gpDraws[1], entry, freq );
            playmode = play;
         }
      }
      skip = 0;
      
      Time( timebuf, vorbis_tell_current(), 0 );
      SOMFontFastString( gpDraw, 232, 32, 
                         config_charset,
                         timebuf,
                         config_color_text,
                         config_color_background ) ;
      
      /* keep loop from rerunning too fast */
      if( repeatcount < config_key_repeat_threshold12 )
      {
         Delay( config_key_repeat_delay1 );
      }
      else
      {
         if( repeatcount < config_key_repeat_threshold23 )
         {
            Delay( config_key_repeat_delay2 );
         }
         else
         {
            Delay( config_key_repeat_delay3 );
         }
      }
      repeatcount++;
      
      if(playmode == seek)
      {
         playmode = play;
      }
      
      do /* do{}while(0) so I can jump out with break */
      {
         /*
          * The new input code: this scares me. It was re-written
          * on a sunday morning in about an hour. First compile
          * had no error. I could also find no runtime problem.
          */
         if( waitrelease )
         {
            GpKeyGetEx( &waitrelease );
            if( waitrelease == key ) break;
            gpShow = gpDraw;
            redraw = 1;
         }
         
         lastkey = key;
         GpKeyGetEx( &key );
         if(lastkey != key)
         {
            repeatcount = 0;
         }
         
         for(function = 0; function < NUMBER_FUNCTIONS; function++)
         {
            if( key &&
                ((config_key[function] & 0x3FF) == key) &&
                (!(status.playmode & PLAYMODE_LOCKED) |
                 (config_key[function] & FUNCTION_OVERRIDE_LOCK) ) )
            {
               switch(function)
               {
                  case FUNCTION_PREVIOUS_TRACK:
                     playmode = stop;
                     vorbis_close();
                     entry = playlist_getprev( entry );
                     if(!entry) entry = playlist_last;
                     dirty = 2;
                     skip = 1;
                     redraw = 1;
                     playmode = next;
                     break;
                  case FUNCTION_NEXT_TRACK:
                     playmode = stop;
                     vorbis_close();
                     entry = playlist_getnext( entry );
                     if(!entry) entry = playlist_first;
                     dirty = 2;
                     skip = 1;
                     redraw = 1;
                     playmode = next;
                     break;
                  case FUNCTION_REWIND:
                     playmode = seek;
                     vorbis_seek( -config_wind_step );
                     skip = 1;
                     break;
                  case FUNCTION_FAST_FORWARD:
                     playmode = seek;
                     vorbis_seek( +config_wind_step );
                     skip = 1;
                     break;
                  case FUNCTION_SEPARATOR_LEFT:
                     if(--status.field1_size < 0)  status.field1_size = 0;
                     redraw = 1;
                     dirty = 1;
                     break;
                  case FUNCTION_SEPARATOR_RIGHT:
                     if(++status.field1_size > 35) status.field1_size = 35;
                     redraw = 1;
                     dirty = 1;
                     break;
                  case FUNCTION_MOVE_TRACK_UP:
                     playlist_swap( entry, -1 );
                     redraw = 1;
                     dirty = 3;
                     skip = 1;
                     break;
                  case FUNCTION_MOVE_TRACK_DOWN:
                     playlist_swap( entry, +1 );
                     redraw = 1;
                     dirty = 3;
                     skip = 1;
                     break;
                  case FUNCTION_RANDOM:
                     switch( config_random_mode )
                     {
                        case RANDOMPLAY:
                           status.playmode ^= PLAYMODE_RANDOM;
                           redraw = 1;
                           waitrelease = 1;
                           break;
                        case SHUFFLE:
                           playlist_shuffle();
                           redraw = 1;
                           dirty = 3;
                           skip = 1;
                           waitrelease = 1;
                           break;
                        default:
                           break;
                     }
                     break;
                  case FUNCTION_PAUSE:
                     playmode = (playmode == play) ? pause : play;
                     DrawStatusLine( gpDraw, 1 );
                     redraw = 1;
                     waitrelease = 1;
                     break;
                  case FUNCTION_LOCK:
                     status.playmode ^= PLAYMODE_LOCKED;
                     redraw = 1;
                     waitrelease = 1;
                     break;
                  case FUNCTION_HELP_SCREEN:
                     gpShow = &gpDraws[2];
                     GpSurfaceFlip( gpShow );
                     waitrelease = 1;
                     break;
                  case FUNCTION_INFO_SCREEN:
                     gpShow = &gpDraws[3];
                     GpRectFill( NULL, &gpDraws[3], 0, 0, gpDraw->buf_w, 16, 
                                 COLOR_CONSOLE_BACKGROUND );
                     SOMFontFastString( &gpDraws[3], 0, 0, config_charset,
                        PROJECT_NAME_LONG " Version " PROJECT_VERSION " System Console",
                        COLOR_CONSOLE_TEXT, COLOR_CONSOLE_BACKGROUND );
                     SOMFontFastString( &gpDraws[3], 0, 8, config_charset,
                        "Compiled: " __DATE__ " " __TIME__,
                        COLOR_CONSOLE_TEXT, COLOR_CONSOLE_BACKGROUND );
                     GpLineDraw( gpShow, 0, 15, 319, 15, COLOR_CONSOLE_LINE );
                     GpSurfaceFlip( gpShow );
                     waitrelease = 1;
                     break;
                  default:
                     break;
               }
               /* found a key, no further check needed */
               break;
            }
         }
      } while(0);
      
      if( playmode == stop )
      {
         int i = 1; /* default: next track */
         
         vorbis_close();
         if( (status.playmode & PLAYMODE_RANDOM) )
         {
            /* only do random if there're more than two tracks */
            if( playlist_total_num > 2 )
            {
               do
               {
                  i = (GpRand() % playlist_total_num);
               }
               while(!i);
            }
         }
         while(i--)
         {
            entry = playlist_getnext( entry );
            if(!entry) entry = playlist_first;
         }
         nextupdate = GpTickCountGet() + 2000;
         dirty      = 0;
         skip       = 0;
         redraw     = 1;
         playmode   = next;
      }
   }
}


void KeyString( char *keystring, int key, int length )
{
   int f=0;
   int b;
   
   for(b=0x200; b; b >>= 1)
   {
      if(b & key)
      {
         switch(b)
         {
            case GPC_VK_SELECT:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "Select", length);
               break;
            case GPC_VK_START:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "Start", length);
               break;
            case GPC_VK_FR:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "R", length);
               break;
            case GPC_VK_FA:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "A", length);
               break;
            case GPC_VK_FB:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "B", length);
               break;
            case GPC_VK_FL:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "L", length);
               break;
            case GPC_VK_UP:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "Up", length);
               break;
            case GPC_VK_RIGHT:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "Right", length);
               break;
            case GPC_VK_DOWN:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "Down", length);
               break;
            case GPC_VK_LEFT:
               if(f) strncat(keystring, "+", length);
               strncat(keystring, "Left", length);
               break;
            default:
               /* shouldn't happen */
               break;
         }
         f=1;
      }
   }
}

void DrawHelpScreen( GPDRAWSURFACE *draw )
{
#define KEYSTRING_LENGTH 24
   const char *messages[] =
   {
    /* 1234567890123456789012345678901234567890 */
      "Written by SvOlli",
      "Special thanks to:",
      "John Willis (for the working SDK),",
      "Yoyo (for idea and example code),",
      "Steffen Boehme (for logo and icon)",
      "and the GP32Xtreme community",
      NULL,
      "",
      "Distributed under the terms of the GPL",
      "All rights reserved, all wrongs revenged",
      "(C) 2004 SvOlli",
      NULL
   };
   const char *function_name[NUMBER_FUNCTIONS] =
   {
      "Previous Track",
      "Next Track",
      "Rewind",
      "Fast Forward",
      "Separator Left",
      "Separator Right",
      "Move Track Up",
      "Move Track Down",
      "Random",
      "Pause",
      "Lock",
      "Help Screen",
      "Info Screen"
   };
   const int function_index[NUMBER_FUNCTIONS] =
   {
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12
   };
   
   char keystring[KEYSTRING_LENGTH];
   int i, m;
   int x, y=32;
   
   DrawOnceOnly( draw, 1 );

   GpLineDraw( draw, 0, y+1, 319, y+1, config_color_lines );
   y+=4;
   for(m=0; messages[m]; m++)
   {
      x=160-strlen(messages[m])*4;
      
      SOMFontDrawString( draw, x, y, config_charset, messages[m],
                         config_color_text,
                         config_color_background, config_color_nontrans );
      y += 8;
   }
   GpLineDraw( draw, 0, y+1, 319, y+1, config_color_lines );
   y+=4;
   
   SOMFontDrawString( draw, 8, y, config_charset, "Key Config: (*=work when locked)",
                      config_color_text,
                      config_color_background, config_color_nontrans );
   y += 12;

   for(i=0; i<NUMBER_FUNCTIONS; i++)
   {
      if(config_key[function_index[i]] & FUNCTION_OVERRIDE_LOCK)
      {
         strcpy(keystring, ":*");
      }
      else
      {
         strcpy(keystring, ": ");
      }
      KeyString(keystring, config_key[function_index[i]], KEYSTRING_LENGTH);
      
      SOMFontDrawString( draw, 8, y, config_charset, function_name[i],
                         config_color_text,
                         config_color_background, config_color_nontrans );
      SOMFontDrawString( draw, 128, y, config_charset, keystring,
                         config_color_text,
                         config_color_background, config_color_nontrans );
      if(i == FUNCTION_RANDOM)
      {
         switch (config_random_mode)
         {
            case SHUFFLE:
               SOMFontDrawString( draw, 8, y, config_charset, "Shuffle",
                                 config_color_text,
                                 config_color_background, config_color_nontrans );
               break;
            default:
               break;
         }
      }
      y += 8;
   }
   
   GpLineDraw( draw, 0, y+1, 319, y+1, config_color_lines );
   y+=4;
   
   for(m++; messages[m]; m++)
   {
      x=160-strlen(messages[m])*4;
      
      SOMFontDrawString( draw, x, y, config_charset, messages[m],
                         config_color_text,
                         config_color_background, config_color_nontrans );
      y += 8;
   }
}



void Setup()
{
   GpLcdEnable();
   GpGraphicModeSet( 8, NULL );
   GpLcdSurfaceGet( &gpDraws[0], 0 );
   GpLcdSurfaceGet( &gpDraws[1], 1 );
   GpLcdSurfaceGet( &gpDraws[2], 2 );
   GpLcdSurfaceGet( &gpDraws[3], 3 );
   gpPalette = GpRegPalGet();
   GpPaletteSelect( gpPalette );
   GpPaletteRealize();
   gpDraw = &gpDraws[0];
   
   GpFatInit();
   
   /* console colors always white on red */
   SOMFontConsoleInit( &gpDraws[3], NULL,
                        COLOR_CONSOLE_TEXT, COLOR_CONSOLE_BACKGROUND );
#ifdef __GNUC__ 
   SOMFontConsolePrint( "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"
                        "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"
                        "compiled with gcc version " __VERSION__ "\n" );
#endif
   config_read();
   GpRectFill( NULL, &gpDraws[0], 0, 0, gpDraw->buf_w, gpDraw->buf_h, config_color_background );
   GpRectFill( NULL, &gpDraws[1], 0, 0, gpDraw->buf_w, gpDraw->buf_h, config_color_background );
   GpRectFill( NULL, &gpDraws[2], 0, 0, gpDraw->buf_w, gpDraw->buf_h, config_color_background );

   DrawOnceOnly( &gpDraws[0], 1 );
   
   gpstorage_init();
   vorbis_init();

   DrawOnceOnly( &gpDraws[0], 0 );
   DrawOnceOnly( &gpDraws[1], 0 );
   DrawHelpScreen( &gpDraws[2] );

   sprintf( conbuf, "CPU set to %dMHz\n", config_cpu_master_speed / 1000000 );
   SOMFontConsolePrint( conbuf );

   snddrv_init();
   GpSrand( GpTickCountGet() );
}



void GpMain( void *arg )
{
   /* to avoid compiler warning */
   arg = arg;

   /* set cpu speed for setting up */
   snddrv_cpuspeed(132000000, 0x3a011, 3);
   GpKeyInit();
   Setup();
   
   /* set cpu speed for playing */
   snddrv_cpuspeed(config_cpu_master_speed, config_cpu_div_factor,
                   config_cpu_clk_mode);
   MainLoop();
   GpAppExit();
}
