
#include <global.h>

#define NUMBUFFERS      32
#define BUFFERSIZE      0x1000
#define MINDISTANCE     8
#define CALLBACKTIMER   40

volatile enum PLAYMODE  playmode;
unsigned int            *snddrv_playpos;
char                    *snddrv_buffer[NUMBUFFERS];
char                    *snddrv_playend;
int                     snddrv_nextbuffer = 0;
int                     snddrv_freq = 0;

/* TODO: on end of ogg wait for data to be played */

static inline int snddrv_getbufferdistance()
{
   char *playpos = (char*)(*snddrv_playpos);
   int relpos;

   relpos = (int)(playpos - snddrv_buffer[0]);
   relpos /= BUFFERSIZE;
   
   if(relpos < snddrv_nextbuffer)
   {
      relpos += NUMBUFFERS;
   }
   
   return (relpos - snddrv_nextbuffer);
}


static inline int snddrv_fillbuffer( char *bufferstart, int rest )
{
   char *buffer = bufferstart;
   int decoded;
   int retval = 0;
   int mute = 0;
   
   snddrv_playend = bufferstart + BUFFERSIZE - 0x20;
   
   if( playmode != play )
   {
      mute = 1;
   }
   
   while( rest > 0 )
   {
      if(mute)
      {
         while( rest > 0 )
         {
            *((int*)buffer) = 0x80008000;
            buffer += 4;
            rest   -= 4;
         }
         decoded = 1;
      }
      else
      {
         decoded = snddrv_decode( buffer, rest );
      }
      
      buffer += decoded;
      rest   -= decoded;
      
      if( decoded < 0 )
      {
         mute = 1;
         retval = decoded;
         playmode = stop;
      }
      
      if( decoded == 0 )
      {
         mute = 1;
         retval = 1;
         playmode = stop;
      }
   }
   return retval;
}


void snddrv_callback()
{
   static int semaphore = 0;
   
   if( semaphore ) return;
   semaphore = 1;

   if( snddrv_getbufferdistance() > MINDISTANCE )
   {
      snddrv_fillbuffer( snddrv_buffer[snddrv_nextbuffer], BUFFERSIZE );
      if(++snddrv_nextbuffer >= NUMBUFFERS)
      {
         snddrv_nextbuffer = 0;
      }
   }

   semaphore = 0;
}


int snddrv_resync( int freq )
{
   int *p;
   int i;
   static int rfreq = -1;
   
   if( snddrv_freq != freq )
   {
      if( snddrv_freq )
      {
         /* wait for buffer to end */
         /* 
         while(snddrv_getbufferdistance());
         while((char*)(*snddrv_playpos) < snddrv_playend);
         */
         GpPcmStop();
      }
      snddrv_freq = freq;
      
      p = (int*)snddrv_buffer[0];
      i = BUFFERSIZE * NUMBUFFERS / 4;
      while(i)
      {
         *p = 0x80008000;
         p++;
         i--;
      }
      snddrv_nextbuffer = 0;
      
      switch( freq )
      {
         case 11025:
            rfreq = GpPcmInit( PCM_S11, PCM_16BIT );
            break;
         case 22050:
            rfreq = GpPcmInit( PCM_S22, PCM_16BIT );
            break;
         case 44100:
            rfreq = GpPcmInit( PCM_S44, PCM_16BIT );
            break;
         default:
            rfreq = -1;
            break;
      }
      
      GpPcmPlay((unsigned short*)snddrv_buffer[0], BUFFERSIZE * NUMBUFFERS, 1);
      GpPcmLock((unsigned short*)snddrv_buffer[0], (int*)&i,
                  (unsigned int*)&snddrv_playpos);  	  	
   }
   
   return rfreq;
}


int snddrv_init()
{
   int i;
   
   snddrv_buffer[0] = (char*)malloc(BUFFERSIZE * NUMBUFFERS);
   if( !snddrv_buffer[0] ) return 1;
   
   for(i = 1; i < NUMBUFFERS; i++)
   {
      snddrv_buffer[i] = snddrv_buffer[i-1] + BUFFERSIZE;
   }
   
   playmode = stop;

   GpTimerOptSet(0, CALLBACKTIMER, 0, snddrv_callback);
   GpTimerSet(0);

   return 0;
}


void snddrv_cpuspeed( int master_speed, int div_factor, int clk_mode )
{
   if(master_speed)
   {
      GpClockSpeedChange(master_speed, div_factor, clk_mode);
   }
}
