
#include <global.h>


ERR_CODE gpstorage_err;
const char *gpstorage_volume = "gp";

/* read playlist and status information */
char *gpstorage_buffer_read( char *filename, ulong *buffersize )
{
   F_HANDLE fh;
   ulong read, size;
   char *buffer;
   
   gpstorage_err = GpFileGetSize( filename, buffersize );
   
   /* open file for reading */
   gpstorage_err = GpFileOpen( config_file_playlist, OPEN_R, &fh );
   
   if( gpstorage_err != SM_OK )
   {
      return NULL;
   }
   
   buffer = (char*)malloc(*buffersize);
   
   for(read = 0; read < *buffersize; read+=size)
   {
      gpstorage_err = GpFileRead( fh, buffer+read, *buffersize-read, &size );
   }
   
   GpFileClose( fh );
   
   return buffer;
}


void gpstorage_buffer_write( char *filename, char *buffer, ulong buffersize )
{
   F_HANDLE fh;
   char *p;
   int i;
   
   /* do all smc stuff at once to minimize write time */
   GpNoFATUpdate(gpstorage_volume);
   gpstorage_err = GpFileCreate( filename, ALWAYS_CREATE, &fh );
   GpFileExtend( fh, buffersize );

   for(p=buffer; p < buffer+buffersize; p+=512)
   {
      /* waste some time to prevent gp32 sound bug */
      for(i=0; i<33333; i++);
      
      /* write only one sector at a time */
      GpFileWrite( fh, p, 512 );
   }
   gpstorage_err = GpFileClose( fh );
   GpFATUpdate(gpstorage_volume);
}


int gpstorage_init()
{
   ulong       dirnum;
   GPDIRENTRY  *dirlist;
   const char  *path = config_ogg_dir;
   int         i;
   int         changed = 0;
   GPFILEATTR  attr;
   char        filename[256];
   int         isfile;
   struct entry_t *entry;

   playlist_total_size = 0;
   playlist_total_num  = 0;
   
   /* step one: read dir */
   gpstorage_err = GpDirEnumNum( path, &dirnum );
   
   dirlist = (GPDIRENTRY*)malloc(dirnum * sizeof(GPDIRENTRY));
   memset((void*)dirlist, 0, dirnum * sizeof(GPDIRENTRY));

   gpstorage_err = GpDirEnumList( path, 0, dirnum, dirlist, &dirnum );

   /* step two: read playlist */
   storage_playlist_read();
   
   /* step three: check for deleted files */
   for(entry = playlist_first; entry; entry = entry->next)
   {
      gpstorage_err = GpFileGetSize( entry->filename, (ulong*)&i );
      if( gpstorage_err != SM_OK )
      {
         playlist_delete( entry );
         SOMFontConsolePrint( entry->filename );
         SOMFontConsolePrint( " removed.\n" );
         changed = 1;
      }
      else
      {
         /* if not deleted, display and add size to total size */
         DrawOncePerTrack( gpDraw, entry, 0 );
         playlist_total_size += entry->size;
         playlist_total_num++;
      }
   }
   
   /* step four: check for new files */
   for(i=0; i< dirnum; i++)
   {
      strcpy(filename,path);
      strcat(filename,"\\");
      strcat(filename,(char*)&dirlist[i]);
      
      GpFileAttr( filename, &attr );
      isfile = !(attr.attr & (1 << 4));
      
      if(isfile)
      {
         if(!storage_inplaylist( filename ))
         {
            entry = playlist_append( NULL, NULL, NULL, 0 );
            if(vorbis_scan( filename, entry ))
            {
               DrawOncePerTrack( gpDraw, entry, 0 );
               playlist_total_size += entry->size;
               playlist_total_num++;
               SOMFontConsolePrint( entry->filename );
               SOMFontConsolePrint( " added.\n" );
               changed = 1;
            }
            else
            {
               playlist_delete( entry );
            }
         }
      }
   }
   sprintf( conbuf, "%2d entries in ", dirnum );
   SOMFontConsolePrint( conbuf );
   SOMFontConsolePrint( path );
   SOMFontConsolePrint( " scanned\n" );
   /* cleanup */
   free(dirlist);

   /* if no Oggs have been found, put out an error message */
   if( !playlist_total_num )
   {
      SOMFontConsolePrint( "No OggVorbis files have been found\nin the directory:\n" );
      SOMFontConsolePrint( path );
      SOMFontConsolePrint( "\n\nPress A To Reboot GP32." );
      GpSurfaceFlip( &gpDraws[3] );
      do
      {
         GpKeyGetEx(&i);
      } while(!(i & GPC_VK_FA));
      GpAppExit();
   }
   
   /* step five: write new playlist */
   if(changed) 
   {
      SOMFontConsolePrint("Playlist changed since last start.\n");
      storage_playlist_write();
   }

   return 0;
}
