
#include <global.h>

struct entry_t *playlist_first = NULL;
struct entry_t *playlist_last  = NULL;
struct entry_t *entry_p        = NULL;
long           playlist_total_size = 0;
long           playlist_total_num  = 0;


char *my_strdup( const char *src )
{
   char *dst = NULL;
   
   if(src)
   {
      dst = (char*)malloc(strlen(src)+1);
      if(dst) strcpy(dst,src);
   }
   return dst;
}


char *my_strndup( const char *src, int maxsize )
{
   char *dst = NULL, *d = NULL;
   int  size = strlen(src);
   int  i;
   char ch;
   
   if(size > maxsize)
   {
      size = maxsize;
   }
   
   if(src)
   {
      dst = (char*)malloc(size+1);
      if(dst)
      {
         d=dst;
         for(i=0;i<size;i++)
         {
            ch = *(src++);
            if(ch == 0xc2)
            {
               /* UTF-8 -> Latin1 conversion first half */
               ch = *(src++);
               if((ch < 0x80) || (ch > 0xbf))
               {
                  src-=2;
                  ch = *(src++);
               }
            }
            else if(ch == 0xc3)
            {
               /* UTF-8 -> Latin1 conversion second half */
               ch = *(src++);
               if((ch < 0x80) || (ch > 0xbf))
               {
                  src-=2;
                  ch = *(src++);
               }
               ch |= 0x40;
            }
            *(d++) = ch;
         }
         *d = 0;
      }
   }
   return dst;
}


struct entry_t *playlist_getprev( struct entry_t* entry )
{
   if(entry) return entry->prev;
   return NULL;
}


struct entry_t *playlist_getnext( struct entry_t* entry )
{
   if(entry) return entry->next;
   return NULL;
}


struct entry_t *playlist_append( char *filename,
   char *artist, char *title, long size )
{
   struct entry_t *entry;
   entry = (struct entry_t*)malloc(sizeof(struct entry_t));
      
   entry->prev     = playlist_last;
   entry->next     = NULL;
   entry->filename = filename;
   entry->artist   = artist;
   entry->title    = title;
   entry->size     = size;
   
   if(!playlist_first)
   {
      playlist_first = entry;
      playlist_last  = entry;
   }
   else
   {
      playlist_last->next = entry;
      entry->prev = playlist_last;
      playlist_last = entry;
   }
   
   return entry;
}


void playlist_remove( struct entry_t *entry )
{
   struct entry_t *prev, *next;
   
   if( entry )
   {
      prev = entry->prev;
      next = entry->next;
      if( prev ) prev->next = next;
      if( next ) next->prev = prev;
      
      if( entry == playlist_first ) playlist_first = entry->next;
      if( entry == playlist_last  ) playlist_last  = entry->prev;
   }
}


void playlist_insert( struct entry_t *entry, struct entry_t *newone )
{
   struct entry_t *next;
      
   if( !newone ) return;
      
   if( entry )
   {
      next = entry->next;
      
      entry->next  = newone;
      newone->next = next;
      newone->prev = entry;
      if( next )
      {
         next->prev = newone;
      }
      else
      {
         playlist_last = newone;
      }
   }
   else
   {
      newone->prev = NULL;
      newone->next = playlist_first;
      playlist_first->prev = newone;
      playlist_first = newone;
   }
}


void playlist_delete( struct entry_t *entry )
{
   if( entry )
   {
      playlist_remove( entry );
      if( entry->filename ) free( entry->filename );
      if( entry->artist   ) free( entry->artist   );
      if( entry->title    ) free( entry->title    );
      free( entry );
      entry = NULL;
   }
}




void playlist_swap( struct entry_t *entry, int direction )
{
   struct entry_t *hook = entry;
   
   if( direction < 0 )
   {
      if(entry->prev)
      {
         hook=entry->prev->prev;
      }
      else
      {
         hook=playlist_last;
      }
   }

   if( direction > 0 )
   {
      hook=entry->next;
   }
   
   playlist_remove( entry );
   playlist_insert( hook, entry );
}


int playlist_getindex( struct entry_t *entry )
{
   struct entry_t *e;
   int i = 0;
      
   for( e = playlist_first; e; e = playlist_getnext(e) )
   {
      if( e == entry ) return i;
      i++;
   }
   return -1;
}


struct entry_t *playlist_getentry( int num )
{
   struct entry_t *e = playlist_first;
   int i;
   
   for( i = 0; i < num; i++ )
   {
      /* sanity check: break if number's too large */
      if( e == playlist_last )
      {
         return playlist_first;
      }
      e = playlist_getnext(e);
   }
   return e;
}


void playlist_recalc()
{
   struct entry_t *e;
   
   playlist_total_size = 0;
   playlist_total_num  = 0;
   for(e = playlist_first; e; e = e->next);
   {
      playlist_total_num++;
      playlist_total_size+=e->size;
   }
}


void playlist_shuffle()
{
   struct entry_t *entry1, *entry2;
   int n,r;
   
   for( n = 0; n < 10 * playlist_total_num; n++ )
   {
      entry1 = playlist_first;
      r = (GpRand() & 0xff) + 1;
      while(r--)
      {
         entry1 = playlist_getnext( entry1 );
         if(!entry1) entry1 = playlist_first;
      }
      playlist_remove( entry1 );
      
      entry2 = playlist_first;
      r = (GpRand() & 0xff) + 1;
      while(r--)
      {
         entry2 = playlist_getnext( entry2 );
         if(!entry1) entry2 = playlist_first;
      }
      playlist_insert( entry2, entry1 );
   }
}
