
#include <global.h>


const unsigned long storage_magic = 0x334c504f;
char *p; /* global read/write pointer */


/* read an integer from data stream */
static inline int storage_read_int()
{
   int value;
   memcpy(&value, p, sizeof(int));
   p += sizeof(int);
   return value;
}

/* write an integer to data stream */
static inline void storage_write_int(int value)
{
   memcpy(p, &value, sizeof(int));
   p += sizeof(int);
}

/* read a string from datastream and allocate memory */
static inline char *storage_read_string()
{
   int length, total;
   char *value = NULL;
   length = storage_read_int();

   if(length % 4)
   {
      total = length + 4 - (length % 4);
   }
   else
   {
      total = length;
   }
   
   if(length >= 0)
   {
      value = (char*)malloc(length+1);
      if(value)
      {
         memcpy(value, p, length);
         value[length] = '\0';
      }
   }
   p += total;
   
   return value;
}

/* write a string to datastream */
static inline void storage_write_string(char *value)
{
   int length, total;
   length = strlen(value);
   storage_write_int(length);
   
   if(length % 4)
   {
      total = length + 4 - (length % 4);
   }
   else
   {
      total = length;
   }
   memset(p, 0, total);
   memcpy(p, value, length);
   
   p += total;
}


/* write playlist and status information */
void storage_playlist_write()
{
   struct entry_t *entry;
   int buffersize, size, i;
   char *buffer;

   /* calculate size */
   buffersize = 3*sizeof(int)+sizeof(struct status_t);
   /* magic + number of entries + status */
   for(entry = playlist_first; entry; entry = entry->next)
   {
      /* 3x stringsize + 1x decoded length */
      buffersize += 4 * sizeof(int);
      buffersize += strlen(entry->filename);
      if(buffersize % 4) buffersize += 4 - (buffersize % 4);
      buffersize += strlen(entry->artist);
      if(buffersize % 4) buffersize += 4 - (buffersize % 4);
      buffersize += strlen(entry->title);
      if(buffersize % 4) buffersize += 4 - (buffersize % 4);
   }
   if(buffersize % 512)
   {
      buffersize += 512 - (buffersize % 512);
   }
   buffer = (char*)malloc(buffersize);
   /*memset(buffer, 0, buffersize);*/
   p = buffer;
   
   /* store magic number */
   /* at end first */
   memcpy(buffer + buffersize - sizeof(int), &storage_magic, sizeof(int));
   /* then at start */
   storage_write_int( storage_magic );
   
   /* store status */
   memcpy(p, &status, sizeof(struct status_t)); p += sizeof(struct status_t);
   
   /* store number of entries */
   size = playlist_getindex( playlist_last ) + 1;
   storage_write_int( size );
   
   /* store entries */
   for(entry = playlist_first; entry; entry = entry->next)
   {
      /* wait a slight bit to keep sound bug down */
      for(i=0; i<3333; i++);
      
      /* store filename */
      storage_write_string( entry->filename );
      
      /* store artist (line 1) */
      if(strlen( entry->artist ) > 40) entry->artist[40] = '\0';
      storage_write_string( entry->artist );
      
      /* store title (line 2) */
      if(strlen( entry->title ) > 40) entry->title[40] = '\0';
      storage_write_string( entry->title );
      
      /* store decoded length of ogg */
      storage_write_int( entry->size );
   }
   
   /* pad with zero bytes */
   memset(p,0,(buffer+buffersize-p-4));

   gpstorage_buffer_write( config_file_playlist, buffer, buffersize);
   
   /* clean up */
   free(buffer);
}


/* read playlist and status information */
void storage_playlist_read()
{
   int i, j;
   char *filename, *artist, *title;
   ulong buffersize;
   char *buffer;
   int size;

   buffer = (char*)gpstorage_buffer_read( config_file_playlist, &buffersize );
   if( (buffersize == 0) || (!buffer) )
   {
      return;
   }
   
   p = buffer;
   
   /* get checksum */
   /* end of file first */
   memcpy( &j, buffer + buffersize - sizeof(int), sizeof(int) );
   /* then start of file */
   i = storage_read_int();
   
   /* only process data if checksums are ok */
   if( (storage_magic == i) && (storage_magic == j) )
   {
      /* read status */
      memcpy(&status, p, sizeof(struct status_t)); p += sizeof(struct status_t);
      
      /* number of playlist entries */
      j = storage_read_int();
      
      for(i = 0; i < j; i++)
      {
         /* collect playlist entry */
         filename = storage_read_string();
         artist   = storage_read_string();
         title    = storage_read_string();
         size     = storage_read_int();
         
         /* create entry from read data */
         playlist_append( filename, artist, title, size );
      }
   }
   
   /* clean up */
   free(buffer);
   
   return;
}


struct entry_t *storage_inplaylist( char *filename )
{
   struct entry_t *entry;
   char *p, *dot;
   
   /* check for ogg extension */
   for(p = filename, dot = filename; *p; p++)
   {
      if(*p == '.') dot = p;
   }
   if(strcasecmp( dot, ".OGG" )) return NULL;
   
   for(entry = playlist_first; entry; entry = entry->next)
   {
      if(!strcmp( entry->filename, filename )) return entry;
   }
   
   return NULL;
}

